<?php
/*
** Copyright (C) 2001-2026 Zabbix SIA
**
** This program is free software: you can redistribute it and/or modify it under the terms of
** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
**
** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU Affero General Public License for more details.
**
** You should have received a copy of the GNU Affero General Public License along with this program.
** If not, see <https://www.gnu.org/licenses/>.
**/


class CControllerSystemWarning extends CController {

	protected function init() {
		$this->disableCsrfValidation();
	}

	protected function checkInput() {
		return $this->validateInput([]);
	}

	protected function checkPermissions() {
		return true;
	}

	protected function doAction() {
		CMessageHelper::restoreScheduleMessages();

		$data = [
			'theme' => getUserTheme(CWebUser::$data),
			'messages' => []
		];

		$data['messages'] = CMessageHelper::getMessages();

		$data['messages'] = array_map(function (array $message): string {
			return $message['message'];
		}, $data['messages']);

		$this->setResponse(new CControllerResponseData($data));
	}
}
