<?php
/**
 * This contians the DuoException class used by
 * Client
 *
 * PHP version 7
 *
 * @category TODO
 * @package  DuoUniversal
 * @author   Duo Security <support@duosecurity.com>
 * @license  https://license_url TODO
 * @link     TODO
 * @file
 */
namespace Duo\DuoUniversal;

/**
 * This class contains a Duo specific exception
 *
 * @category TODO
 * @package  DuoUniversal
 * @author   Duo Security <support@duosecurity.com>
 * @license  https://license_url TODO
 * @link     TODO
 */
class DuoException extends \Exception
{
}
