<?php return array(
    'root' => array(
        'name' => 'zabbix/ui',
        'pretty_version' => 'dev-master',
        'version' => 'dev-master',
        'reference' => 'e4ff6665babd322e51560ce4575dd8fc1f419ab3',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        'duosecurity/duo_universal_php' => array(
            'pretty_version' => '1.1.2',
            'version' => '1.1.2.0',
            'reference' => '68e699c52d41f681df07dc0f98e2ba551bef2fd3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../duosecurity/duo_universal_php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'firebase/php-jwt' => array(
            'pretty_version' => 'v7.0.3',
            'version' => '7.0.3.0',
            'reference' => '28aa0694bcfdfa5e2959c394d5a1ee7a5083629e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../firebase/php-jwt',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'onelogin/php-saml' => array(
            'pretty_version' => '4.3.1',
            'version' => '4.3.1.0',
            'reference' => 'b009f160e4ac11f49366a45e0d45706b48429353',
            'type' => 'library',
            'install_path' => __DIR__ . '/../onelogin/php-saml',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/constant_time_encoding' => array(
            'pretty_version' => 'v2.6.3',
            'version' => '2.6.3.0',
            'reference' => '58c3f47f650c94ec05a151692652a868995d2938',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/constant_time_encoding',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'pragmarx/google2fa' => array(
            'pretty_version' => 'v8.0.1',
            'version' => '8.0.1.0',
            'reference' => '80c3d801b31fe165f8fe99ea085e0a37834e1be3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../pragmarx/google2fa',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'robrichards/xmlseclibs' => array(
            'pretty_version' => '3.1.4',
            'version' => '3.1.4.0',
            'reference' => 'bc87389224c6de95802b505e5265b0ec2c5bcdbd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../robrichards/xmlseclibs',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/deprecation-contracts' => array(
            'pretty_version' => 'v2.1.3',
            'version' => '2.1.3.0',
            'reference' => '5e20b83385a77593259c9f8beb2c43cd03b2ac14',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/deprecation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.18.1',
            'version' => '1.18.1.0',
            'reference' => '1c302646f6efc070cd46856e600e5e0684d6b454',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/yaml' => array(
            'pretty_version' => 'v5.1.3',
            'version' => '5.1.3.0',
            'reference' => 'ea342353a3ef4f453809acc4ebc55382231d4d23',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/yaml',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'zabbix/ui' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'reference' => 'e4ff6665babd322e51560ce4575dd8fc1f419ab3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
